/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSet_PointCoordIterator.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/*=======================================================================*/
// PointCoordIterator is a general use PointCoordIterator for bitmap sets
// It behaves similarly to the PointIterator but
// unlike the previous operators it stores and updates the
// cooridnates of the point

#if !defined(_EUCLBITSETT_POINTCOORDITERATOR_H_)
#define _EUCLBITSETT_POINTCOORDITERATOR_H_

template <typename P_BitSet,int dim>
class EuclBitSetT<P_BitSet,dim>::PointCoordIterator : public EuclBitSetT<P_BitSet,dim>::BitCoordIterator{
  friend class EuclBitSetT<P_BitSet,dim>;
private:
  BitIterator& incInDir(int i);
  BitIterator& decInDir(int i);
protected:
  static const int bitsPerWord = 8*sizeof(typename EuclBitSetT<P_BitSet,dim>::Word);

  bool incCoord(int n){ // increase by n
    for(int k=0;k<dim;++k){
      if( (this->coord[k]+=n) < this->baseEuclBitSet()->paddedBitWidth[k] ) return true;
      n = this->coord[k] / this->baseEuclBitSet()->paddedBitWidth[k];
      this->coord[k] %= this->baseEuclBitSet()->paddedBitWidth[k];
    }
    return false;
  }

  bool moveToFirstPixel(){
    Word* c=this->wIt;
    int b=this->bit;
    this->findPoint();
    int n=(this->wIt - c)*bitsPerWord + (this->bit - b);
    this->incCoord(n);
    return this->wIt < this->baseEuclBitSet()->bitmapEnd;
  }

public:

  PointCoordIterator(const EuclBitSetT& s):BitCoordIterator(s){
    for(int k=0;k<dim;k++) this->coord[k]=0;
    this->moveToFirstPixel();
  }

  // intended as a converter
  PointCoordIterator(const BitIterator& A_it): BitCoordIterator(A_it){
    this->moveToFirstPixel();
//    int pos=this->wIt - this->itSet->bitmap;
    int pos=this->wIt - this->baseEuclBitSet()->getBitmap();
    for(int i=0;i<dim;++i){
      this->coord[i]=pos % this->baseEuclBitSet()->wordWidth[i];
      pos /= this->baseEuclBitSet()->wordWidth[i];
    }
    this->coord[0]*=bitsPerWord;
    this->coord[0]+=this->bit;
  }

  // Keep it here! Do not inherit
  PointCoordIterator& operator++(){
    EuclBitSetT<P_BitSet,dim>::BitIterator::operator++();
    this->EuclBitSetT::BitCoordIterator::incCoord();
    this->moveToFirstPixel();
    return *this;
  }

};
#endif //_EUCLBITSETT_POINTCOORDITERATOR_H_

/*=======================================================================*/
/// @}

